@@ -63,19 +63,11 @@ struct _EogMetadataReaderPrivate {
 	int      bytes_read;	
 };
 
+#define EOG_METADATA_READER_GET_PRIVATE(object) \
+	(G_TYPE_INSTANCE_GET_PRIVATE ((object), EOG_TYPE_METADATA_READER, EogMetadataReaderPrivate))
+
 G_DEFINE_TYPE (EogMetadataReader, eog_metadata_reader, G_TYPE_OBJECT)
 
-static void
-eog_metadata_reader_finalize (GObject *object)
-{
-	EogMetadataReader *instance = EOG_METADATA_READER (object);
-	
-	g_free (instance->priv);
-	instance->priv = NULL;
-
-	G_OBJECT_CLASS (eog_metadata_reader_parent_class)->finalize (object);
-}
-
 static void
 eog_metadata_reader_dispose (GObject *object)
 {
@@ -109,15 +101,13 @@ eog_metadata_reader_init (EogMetadataReader *obj)
 {
 	EogMetadataReaderPrivate *priv;
 
-	priv = g_new0 (EogMetadataReaderPrivate, 1);
+	priv = obj->priv =  EOG_METADATA_READER_GET_PRIVATE (obj);
 	priv->exif_chunk = NULL;
 	priv->exif_len = 0;
 	priv->iptc_chunk = NULL;
 	priv->iptc_len = 0;
 	priv->icc_chunk = NULL;
 	priv->icc_len = 0;
-	
-	obj->priv = priv;
 }
 
 static void 
@@ -125,8 +115,9 @@ eog_metadata_reader_class_init (EogMetadataReaderClass *klass)
 {
 	GObjectClass *object_class = (GObjectClass*) klass;
 
-	object_class->finalize = eog_metadata_reader_finalize;
 	object_class->dispose = eog_metadata_reader_dispose;
+
+	g_type_class_add_private (klass, sizeof (EogMetadataReaderPrivate));
 }
 
 EogMetadataReader*
@@ -287,7 +278,6 @@ eog_metadata_reader_consume (EogMetadataReader *emr, guchar *buf, guint len)
 			
 			EogMetadataReaderState next_state;
 			guchar *chunk = NULL;
-			int offset = 0;
 
 			switch (app1_type) {
 			case EJA_EXIF:
@@ -301,14 +291,11 @@ eog_metadata_reader_consume (EogMetadataReader *emr, guchar *buf, guint len)
 				break;
 			case EJA_XMP:
 				if (priv->xmp_chunk == NULL) { 
-					offset = 29 + 54; /* skip the ID + packet */
-						if (priv->size > offset)  { /* ensure that we have enough bytes */
-						priv->xmp_chunk = g_new0 (guchar, priv->size);
-						priv->xmp_len = priv->size - offset;
-						priv->bytes_read = 0;
-						chunk = priv->xmp_chunk;
-						next_state = EMR_READ_XMP;
-					}
+					priv->xmp_chunk = g_new0 (guchar, priv->size);
+					priv->xmp_len = priv->size;
+					priv->bytes_read = 0;
+					chunk = priv->xmp_chunk;
+					next_state = EMR_READ_XMP;
 				}
 				break;
 			case EJA_OTHER:
@@ -319,19 +306,10 @@ eog_metadata_reader_consume (EogMetadataReader *emr, guchar *buf, guint len)
 			}
 
 			if (chunk) {
-				if (i + priv->size < len) {
-					/* read data in one block */
-					memcpy ((guchar*) (chunk) + priv->bytes_read, &buf[i + offset], priv->size);
-					priv->state = EMR_READ;
-					i = i + priv->size - 1; /* the for-loop consumes the other byte */
-				} else {
-					int chunk_len = len - i;
-					memcpy ((guchar*) (priv->exif_chunk) + priv->bytes_read, &buf[i], chunk_len);
-					priv->bytes_read += chunk_len; /* bytes already read */
-					priv->size = (i + priv->size) - len; /* remaining data to read */
-					i = len - 1;
-					priv->state = next_state;
-				}
+				eog_metadata_reader_get_next_block (priv, chunk,
+								    &i, buf,
+								    len,
+								    next_state);
 			}
 
 			if (IS_FINISHED(priv))
@@ -359,7 +337,9 @@ eog_metadata_reader_consume (EogMetadataReader *emr, guchar *buf, guint len)
 			break;
 			
 		case EMR_READ_ICC:			
-			eog_debug_message (DEBUG_IMAGE_DATA, "Read ICC data, Length: %i\n", priv->size);
+			eog_debug_message (DEBUG_IMAGE_DATA,
+					   "Read continuation of ICC data, "
+					   "length: %i", priv->size);
 
 			if (priv->icc_chunk == NULL) { 
 				priv->icc_chunk = g_new0 (guchar, priv->size);
@@ -375,25 +355,20 @@ eog_metadata_reader_consume (EogMetadataReader *emr, guchar *buf, guint len)
 			break;
 			
 		case EMR_READ_IPTC:
+			eog_debug_message (DEBUG_IMAGE_DATA,
+					   "Read continuation of IPTC data, "
+					   "length: %i", priv->size);
+
 			if (priv->iptc_chunk == NULL) { 
 				priv->iptc_chunk = g_new0 (guchar, priv->size);
 				priv->iptc_len = priv->size;
 				priv->bytes_read = 0;
 			}
 
-			if (i + priv->size < len) {
-                                /* read data in one block */
-				memcpy ((guchar*) (priv->iptc_chunk) + priv->bytes_read, &buf[i], priv->size);
-				i = i + priv->size - 1; /* the for-loop consumes the other byte */
-				priv->state = EMR_READ;
-			} else {
-				int chunk_len = len - i;
-				memcpy ((guchar*) (priv->iptc_chunk) + priv->bytes_read, &buf[i], chunk_len);
-				priv->bytes_read += chunk_len; /* bytes already read */
-				priv->size = (i + priv->size) - len; /* remaining data to read */
-				i = len - 1;
-				priv->state = EMR_READ_IPTC;
-			}
+			eog_metadata_reader_get_next_block (priv,
+							    priv->iptc_chunk,
+							    &i, buf, len,
+							    EMR_READ_IPTC);
 			
 			if (IS_FINISHED(priv))
 				priv->state = EMR_FINISHED;
@@ -443,6 +418,10 @@ eog_metadata_reader_get_exif_data (EogMetadataReader *emr)
 
 
 #ifdef HAVE_EXEMPI
+
+/* skip the ID + packet */
+#define EOG_XMP_OFFSET (29 + 54)
+
 XmpPtr 
 eog_metadata_reader_get_xmp_data (EogMetadataReader *emr )
 {
@@ -454,7 +433,8 @@ eog_metadata_reader_get_xmp_data (EogMetadataReader *emr )
 	priv = emr->priv;
 
 	if (priv->xmp_chunk != NULL) {
-		xmp = xmp_new (priv->xmp_chunk, priv->xmp_len);
+		xmp = xmp_new (priv->xmp_chunk+EOG_XMP_OFFSET,
+			       priv->xmp_len-EOG_XMP_OFFSET);
 	}
 
 	return xmp;
